use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Alders'
Atomic
CopyFiles
CopyFooBarFilesToProduction
DBI
DSL
DSN
Dave
Eilam
Eilam's
Error
FileGenerator
FinalStep
Fowler
GitHub
Graph
GraphBuilder
Greg
Inc
Internal
LoggerWithMoniker
MAXMIND
MAXMIND's
Mark
MaxMind
MaxMind's
Olaf
Oschwald
Oschwald's
PayPal
Ran
Role
Rolsky
Rolsky's
Runner
Step
StepDependency
StepProduction
Stepfile
Stepford
Trait
Types
Unserializable
VERSIONING
YYY
YYYZZZ
ZZZ
docs
drolsky
goschwald
iteratively
lib
mfowler
moniker
namespace
namespaces
oalders
prepends
reilam
serializable
subtypes
timestamp
timestamps
unserializable
username
versioning
